@@ -15,8 +15,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
- *
- *  $Id$
  */
 
 #include "config.h"
@@ -1436,7 +1434,7 @@ aisleriot_game_set_paused (AisleriotGame *game,
   if (paused == game->paused)
     return;
 
-  paused = game->paused;
+  game->paused = paused;
 
   /* (Re)store the start time */
   game->start_time = time (NULL) - game->start_time;
