@@ -28,6 +28,44 @@ f3 Bc8 34. Kf2 Bf5 35. Ra7 g6 36. Ra6+ Kc5 37. Ke1 Nf4 38. g3 Nxh3 39. Kd2 Kb5
 40. Rd6 Kc5 41. Ra6 Nf2 42. g4 Bd3 43. Re6 1/2-1/2
 """
 
+RESULT_INCOMPLETE = '*'
+RESULT_WHITE_WIN  = '1-0'
+RESULT_BLACK_WIN  = '0-1'
+RESULT_DRAW       = '1/2-1/2'
+
+"""The required tags in a PGN file (the seven tag roster, STR)"""
+TAG_EVENT  = 'Event'
+TAG_SITE   = 'Site'
+TAG_DATE   = 'Date'
+TAG_ROUND  = 'Round'
+TAG_WHITE  = 'White'
+TAG_BLACK  = 'Black'
+TAG_RESULT = 'Result'
+
+"""Optional tags"""
+TAG_TIME         = 'Time'
+TAG_FEN          = 'FEN'
+TAG_WHITE_TYPE   = 'WhiteType'
+TAG_WHITE_ELO    = 'WhiteElo'
+TAG_BLACK_TYPE   = 'BlackType'
+TAG_BLACK_ELO    = 'BlackElo'
+TAG_TIME_CONTROL = 'TimeControl'
+TAG_TERMINATION  = 'Termination'
+
+# Values for the WhiteType and BlackType tag
+PLAYER_HUMAN     = 'human'
+PLAYER_AI        = 'program'
+
+# Values for the Termination tag
+TERMINATE_ABANDONED        = 'abandoned'
+TERMINATE_ADJUDICATION     = 'adjudication'
+TERMINATE_DEATH            = 'death'
+TERMINATE_EMERGENCY        = 'emergency'
+TERMINATE_NORMAL           = 'normal'
+TERMINATE_RULES_INFRACTION = 'rules infraction'
+TERMINATE_TIME_FORFEIT     = 'time forfeit'
+TERMINATE_UNTERMINATED     = 'unterminated'
+
 # Comments are bounded by ';' to '\n' or '{' to '}'
 # Lines starting with '%' are ignored and are used as an extension mechanism
 # Strings are bounded by '"' and '"' and quotes inside the strings are escaped with '\"'
@@ -40,17 +78,13 @@ class Error(Exception):
     # Text description of the error
     description = ''
     
-    # The Unix error number
-    errno = -1
-    
     # The file being opened
     fileName = ''
     
-    def __init__(self, description = '', errno = -1, fileName = ''):
+    def __init__(self, description = '', fileName = ''):
         """
         """
         self.description = description
-        self.errno = errno
         self.fileName = fileName
         Exception.__init__(self)
         
@@ -59,8 +93,6 @@ class Error(Exception):
             string = self.fileName + ': '
         else:
             string = ''
-        if self.errno >= 0:
-            string += '[Error %i] ' % self.errno
         string += self.description
         return string
 
@@ -85,14 +117,9 @@ class PGNToken:
     type = None
 
     SYMBOL_START_CHARACTERS = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ' + '*'
-    SYMBOL_CONTINUATION_CHARACTERS = SYMBOL_START_CHARACTERS + '_+#=:-' + '/' # Not in spec but required from game draw and imcomplete
+    SYMBOL_CONTINUATION_CHARACTERS = SYMBOL_START_CHARACTERS + '_+#=:-' + '/' # Not in spec but required from game draw and incomplete
     NAG_CONTINUATION_CHARACTERS = '0123456789'
     
-    GAME_TERMINATE_INCOMPLETE = '*'
-    GAME_TERMINATE_WHITE_WIN  = '1-0'
-    GAME_TERMINATE_BLACK_WIN  = '0-1'
-    GAME_TERMINATE_DRAW       = '1/2-1/2'
-
     data = None
     
     lineNumber = -1
@@ -112,6 +139,9 @@ class PGNToken:
             string += ': ' + self.data
         return string
     
+    def __repr__(self):
+        return self.__str__()
+    
 class PGNParser:
     """
     """
@@ -120,6 +150,58 @@ class PGNParser:
     __comment = ''
     __startOffset = -1
     
+    def __init__(self):
+        self.tokens = {' ':  (None,                  1),
+                       '\t': (None,                  1),
+                       '\n': (None,                  1),
+                       ';':  (PGNToken.LINE_COMMENT, self.__lineComment),
+                       '{':  (PGNToken.LINE_COMMENT, self.__collectComment),
+                       '.':  (PGNToken.PERIOD,       1),
+                       '[':  (PGNToken.TAG_START,    1),
+                       ']':  (PGNToken.TAG_END,      1),
+                       '"':  (PGNToken.STRING,       self.__extractPGNString),
+                       '(':  (PGNToken.RAV_START,    1),
+                       ')':  (PGNToken.RAV_END,      1),
+                       '<':  (PGNToken.XML_START,    1),
+                       '>':  (PGNToken.XML_END,      1),
+                       '$':  (PGNToken.NAG,          self.__extractNAG),
+                       '*':  (PGNToken.SYMBOL,       1)}
+
+        for c in '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ':
+            self.tokens[c] = (PGNToken.SYMBOL, self.__extractSymbol)
+            
+    def __lineComment(self, data):
+        return (data, len(data))
+
+    def __collectComment(self, data):
+        index = data.find('}')
+        if index < 0:
+           self.__inComment = True
+           index = data.find('\n')
+           self.__comment = self.__comment + data[:index+1]
+           return (data[:index+1], index+1)
+        else:
+            self.__comment = self.__comment + data[:index+1]
+            self.__inComment = False
+                 
+        return (self.__comment, index+1)
+            
+    def __extractSymbol(self, data):
+        for offset in xrange(1, len(data)):
+            if PGNToken.SYMBOL_CONTINUATION_CHARACTERS.find(data[offset]) < 0:
+                return (data[:offset], offset)
+
+        return (data, offset)
+            
+    def __extractNAG(self, data):
+        index = PGNToken.NAG_CONTINUATION_CHARACTERS.find(data[1])
+        for offset in xrange(1, len(data)):            
+            if PGNToken.NAG_CONTINUATION_CHARACTERS.find(data[offset]) < 0:
+            	#FIXME: Should be at lest one character and less than $255
+                return (data[:offset], offset)
+
+        return (data, offset)
+    
     def __extractPGNString(self, data):
         #"""Extract a PGN string.
         
@@ -132,20 +214,17 @@ class PGNParser:
         if data[0] != '"':
             raise Error('PGN string does not start with "')
         
-        offset = 1
-        escaped = False
-        while True:
-            try:
-                c = data[offset]
-                escaped = (c == '\\')
-                if c == '"' and escaped is False:
-                    pgnString = data[1:offset]
-                    pgnString.replace('\\"', '"')
-                    return (pgnString, offset + 1)
-            except IndexError:
-                raise Error('Unterminated PGN string')
-            offset += 1
-    
+        for offset in xrange(1, len(data)):
+            c = data[offset]
+            escaped = (c == '\\')
+            if c == '"' and escaped is False:
+                pgnString = data[1:offset]
+                pgnString.replace('\\"', '"')
+                pgnString.replace('\\\\', '\\')
+                return (pgnString, offset + 1)
+
+        raise Error('Unterminated PGN string')
+            
     def parseLine(self, line, lineNumber):
         """TODO
         
@@ -155,89 +234,34 @@ class PGNParser:
         if line[0] == '%':
             return [PGNToken(lineNumber, self.__startOffset, PGNToken.ESCAPED, line[1:])]
         
-        tokens = []
-        inSymbol = False
-        inNAG = False
-        symbol = ''
-        nag = ''
         offset = 0
+        tokens = []
         while offset < len(line):
             c = line[offset]
-            
-            if self.__inComment is True:
-                if c == '}':
-                    tokens.append(PGNToken(lineNumber, self.__startOffset, PGNToken.LINE_COMMENT, self.__comment))
-                    self.__inComment = False
-                else:
-                    self.__comment += c
-                offset += 1
-                continue
-            
-            if inSymbol:
-                if PGNToken.SYMBOL_CONTINUATION_CHARACTERS.find(c) >= 0:
-                    symbol += c
-                    offset += 1
-                    continue
-                else:
-                    tokens.append(PGNToken(lineNumber, self.__startOffset, PGNToken.SYMBOL, symbol))
-                    inSymbol = False
-            elif inNAG:
-                if PGNToken.NAG_CONTINUATION_CHARACTERS.find(c) >= 0:
-                    symbol += c
-                    offset += 1
-                    continue
-                else:
-                    # FIXME: Should be at least one character
-                    tokens.append(PGNToken(lineNumber, self.__startOffset, PGNToken.NAG, nag))
-                    inNAG = False
-            
-            if c.isspace():
-                pass
-            elif c == ';':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.LINE_COMMENT, line[offset:]))
-                return tokens
-            elif c == '{':
-                self.__comment = ''
-                self.__inComment = True
-                self.__startOffset = offset
-            elif c == '.':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.PERIOD))
-            elif c == '[':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.TAG_START))
-            elif c == ']':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.TAG_END))
-            elif c == '"':
-                (string, newOffset) = self.__extractPGNString(line[offset:])
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.STRING, string))
-                offset += newOffset
-                continue
-            elif c == '(':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.RAV_START))
-            elif c == ')':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.RAV_END))
-            elif c == '<':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.XML_START))
-            elif c == '>':
-                tokens.append(PGNToken(lineNumber, offset+1, PGNToken.XML_END))
-            elif c == '$':
-                inNAG = True
-                self.__startOffset = offset
-            elif PGNToken.SYMBOL_START_CHARACTERS.find(c) >= 0:
-                symbol = c
-                inSymbol = True
-                self.__startOffset = offset
+
+            if self.__inComment:
+                try:
+                    (tokenType, length) = self.tokens['{']
+                except KeyError:
+                    raise Error('Unknown character %s' % repr(c))
             else:
-                raise Error('Unknown character %s' % repr(c))
+                self.__comment = ''
+                try:
+                    (tokenType, length) = self.tokens[c]
+                except KeyError:
+                    raise Error('Unknown character %s' % repr(c))
 
-            offset += 1
-
-        # Complete any symbols or NAGs
-        if inSymbol:
-            tokens.append(PGNToken(lineNumber, self.__startOffset+1, PGNToken.SYMBOL, symbol))
-        if inNAG:
-            # FIXME: Must be 1 or more char..
-            tokens.append(PGNToken(lineNumber, self.__startOffset+1, PGNToken.NAG, nag))
+            startOffset = offset
+            if type(length) is int:
+                data = line[offset:offset+length]
+                offset += length
+            else:
+                (data, o) = length(line[offset:])
+                offset += o
             
+            if tokenType is not None and not self.__inComment:
+                tokens.append(PGNToken(lineNumber, startOffset, tokenType, data))
+
         return tokens
             
     def endParse(self):
@@ -265,8 +289,10 @@ class PGNGameParser:
     
     # The move number being decoded
     __expectedMoveNumber = 0
-    __lastTokenIsMoveNumber = False
+    __prevTokenIsMoveNumber = False
     
+    __currentMoveNumber = 0
+
     # The last white move
     __whiteMove = None
     
@@ -291,19 +317,15 @@ class PGNGameParser:
             return
                 
         if token.type is PGNToken.PERIOD:
-            if self.__lastTokenIsMoveNumber is False:
+            if self.__prevTokenIsMoveNumber is False:
                 raise Error('Unexpected period on line %i:%i' % (token.lineNumber, token.characterNumber))
 
         elif token.type is PGNToken.SYMBOL:
             # See if this is a game terminate
-            if token.data == PGNToken.GAME_TERMINATE_INCOMPLETE or \
-               token.data == PGNToken.GAME_TERMINATE_WHITE_WIN or \
-               token.data == PGNToken.GAME_TERMINATE_BLACK_WIN or \
-               token.data == PGNToken.GAME_TERMINATE_DRAW:
-                # Complete any half moves
-                if self.__whiteMove is not None:
-                    self.__game.addMove(self.__whiteMove, None)
-
+            if token.data == RESULT_INCOMPLETE or \
+               token.data == RESULT_WHITE_WIN or \
+               token.data == RESULT_BLACK_WIN or \
+               token.data == RESULT_DRAW:
                 game = self.__game
                 self.__game = None
                 
@@ -314,17 +336,28 @@ class PGNGameParser:
                 # See if this is a move number or a SAN move
                 try:
                     moveNumber = int(token.data)
-                    self.__lastTokenIsMoveNumber = True
-                    if moveNumber != self.__expectedMoveNumber:
-                        raise Error('Expected move number %i, got %i on line %i:%i' % (self.__expectedMoveNumber, moveNumber, token.lineNumber, token.characterNumber))
                 except ValueError:
-                    self.__lastTokenIsMoveNumber = False
+                    self.__prevTokenIsMoveNumber = False
                     if self.__whiteMove is None:
                         self.__whiteMove = token.data
+                        move = PGNMove()
+                        move.number = self.__currentMoveNumber
+                        move.move = self.__whiteMove
+                        self.__game.addMove(move)
+                        self.__currentMoveNumber += 1
+
                     else:
-                        self.__game.addMove(self.__whiteMove, token.data)
+                        move = PGNMove()
+                        move.move = token.data
+                        move.number = self.__currentMoveNumber
+                        self.__game.addMove(move)
                         self.__whiteMove = None
+                        self.__currentMoveNumber += 1
                         self.__expectedMoveNumber += 1
+                else:
+                    self.__prevTokenIsMoveNumber = True
+                    if moveNumber != self.__expectedMoveNumber:
+                        raise Error('Expected move number %i, got %i on line %i:%i' % (self.__expectedMoveNumber, moveNumber, token.lineNumber, token.characterNumber))
 
         elif token.type is PGNToken.NAG:
             pass
@@ -337,9 +370,17 @@ class PGNGameParser:
         
         Return a game object if a game is complete otherwise None.
         """
-        
         # Ignore all comments at any time
         if token.type is PGNToken.LINE_COMMENT or token.type is PGNToken.COMMENT:
+            if self.__currentMoveNumber > 0:
+                move = self.__game.getMove(self.__currentMoveNumber)
+                move.comment = token.data[1:-1]
+            return None
+
+        if token.type is PGNToken.NAG:
+            if self.__currentMoveNumber > 0:
+                move = self.__game.getMove(self.__currentMoveNumber)
+                move.nag = token.data               
             return None
             
         if self.__state is self.STATE_IDLE:
@@ -353,7 +394,7 @@ class PGNGameParser:
             elif token.type is PGNToken.SYMBOL:
                 self.__expectedMoveNumber = 1
                 self.__whiteMove = None
-                self.__lastTokenIsMoveNumber = False
+                self.__prevTokenIsMoveNumber = False
                 self.__ravDepth = 0
                 self.__state = self.STATE_MOVETEXT
                 
@@ -362,7 +403,7 @@ class PGNGameParser:
             
             else:
                 raise Error('Unexpected token %s on line %i:%i' % (str(token.type), token.lineNumber, token.characterNumber))
-            
+
         if self.__state is self.STATE_TAG_NAME:
             if token.type is PGNToken.SYMBOL:
                 self.__tagName = token.data
@@ -383,7 +424,7 @@ class PGNGameParser:
                 self.__state = self.STATE_IDLE
             else:
                 raise Error('Not a valid file')
-                    
+
         elif self.__state is self.STATE_MOVETEXT:
             game = self.__parseTokenMovetext(token)
             if game is not None:
@@ -393,25 +434,27 @@ class PGNGameParser:
     def complete(self):
         """
         """
-        pass
-        # Raise an error if there was a partial game
-        #raise Error()
+        self.__state = self.STATE_IDLE
+        return self.__game
+        
+class PGNMove:
+    """
+    """
+    #
+    move       = ''
     
+    #
+    comment    = ''
+
+    #
+    nag        = ''
+
 class PGNGame:
     """
     """
-    
-    """The required tags in a PGN file (the seven tag roster, STR)"""
-    PGN_TAG_EVENT  = 'Event'
-    PGN_TAG_SITE   = 'Site'
-    PGN_TAG_DATE   = 'Date'
-    PGN_TAG_ROUND  = 'Round'
-    PGN_TAG_WHITE  = 'White'
-    PGN_TAG_BLACK  = 'Black'
-    PGN_TAG_RESULT = 'Result'
-    
+
     # The seven tag roster in the required order (REFERENCE)
-    __strTags = [PGN_TAG_EVENT, PGN_TAG_SITE, PGN_TAG_DATE, PGN_TAG_ROUND, PGN_TAG_WHITE, PGN_TAG_BLACK, PGN_TAG_RESULT]
+    __strTags = [TAG_EVENT, TAG_SITE, TAG_DATE, TAG_ROUND, TAG_WHITE, TAG_BLACK, TAG_RESULT]
 
     # The tags in this game
     __tagsByName = None
@@ -421,13 +464,13 @@ class PGNGame:
     def __init__(self):
         # Set the default STR tags
         self.__tagsByName = {}
-        self.setTag(self.PGN_TAG_EVENT, '?')
-        self.setTag(self.PGN_TAG_SITE, '?')
-        self.setTag(self.PGN_TAG_DATE, '????.??.??')
-        self.setTag(self.PGN_TAG_ROUND, '?')
-        self.setTag(self.PGN_TAG_WHITE, '?')
-        self.setTag(self.PGN_TAG_BLACK, '?')
-        self.setTag(self.PGN_TAG_RESULT, '*')
+        self.setTag(TAG_EVENT, '?')
+        self.setTag(TAG_SITE, '?')
+        self.setTag(TAG_DATE, '????.??.??')
+        self.setTag(TAG_ROUND, '?')
+        self.setTag(TAG_WHITE, '?')
+        self.setTag(TAG_BLACK, '?')
+        self.setTag(TAG_RESULT, '*')
         
         self.__moves = []
         
@@ -447,16 +490,19 @@ class PGNGame:
         
         # Insert numbers in-between moves
         tokens = []
-        moveNumber = 1
+        moveNumber = 0
         for m in self.__moves:
-            tokens.append('%i.' % moveNumber)
+            if moveNumber % 2 == 0:
+                tokens.append('%i.' % (moveNumber / 2 + 1))
             moveNumber += 1
-            tokens.append(m[0])
-            if m[1] is not None:
-                tokens.append(m[1])
+            tokens.append(m.move)
+            if m.nag != '':
+                tokens.append(m.nag)
+            if m.comment != '':
+                tokens.append('{' + m.comment + '}')
                 
         # Add result token to the end
-        tokens.append(self.__tagsByName[self.PGN_TAG_RESULT])
+        tokens.append(self.__tagsByName[TAG_RESULT])
 
         # Print moves keeping the line length to less than 256 characters (PGN requirement)
         line = ''
@@ -520,22 +566,14 @@ class PGNGame:
         except KeyError:
             return default
         
-    def addMove(self, whiteMove, blackMove):
-        self.__moves.append((whiteMove, blackMove))
-        
-    def getWhiteMove(self, moveNumber):
-        return self.__moves[moveNumber - 1][0]
-    
-    def getBlackMove(self, moveNumber):
-        return self.__moves[moveNumber - 1][1]
+    def addMove(self, move):
+        self.__moves.append(move)
+
+    def getMove(self, moveNumber):
+        return self.__moves[moveNumber - 1]
     
     def getMoves(self):
-        moves = []
-        for m in self.__moves:
-            moves.append(m[0])
-            if m[1] is not None:
-                moves.append(m[1])
-        return moves
+        return self.__moves
 
     def __str__(self):
         
@@ -623,10 +661,7 @@ class PGN:
         
         'fileName' is the name of the file to save to.
         """
-        try:
-            f = file(fileName, 'w')
-        except IOError, e:
-            raise Error(e.args[1], errno = e.args[0])
+        f = file(fileName, 'w')
         # FIXME: Set the newline characters to the correct type?
         
         # Sign it from glChess
@@ -655,10 +690,7 @@ class PGN:
         """
         """
         # Convert the file into PGN tokens
-        try:
-            f = file(fileName, 'r')
-        except IOError, e:            
-            raise Error(e.args[1], errno = e.args[0])
+        f = file(fileName, 'r')
         p = PGNParser()
         gp = PGNGameParser()
         lineNumber = 1
@@ -667,8 +699,11 @@ class PGN:
             # Read a line from the file
             line = f.readline()
             if line == '':
+                game = gp.complete()
+                if game is not None:
+                    self.__games.append(game)
                 break
-            
+
             # Parse the line into tokens
             tokens = p.parseLine(line, lineNumber)
 
@@ -678,12 +713,13 @@ class PGN:
                 
                 # Store this game and stop if only required to parse a certain number
                 if game is not None:
+                    gp = PGNGameParser()
                     self.__games.append(game)
                     gameCount += 1
 
                     if maxGames is not None and gameCount >= maxGames:
                         break
-                    
+
             # YUCK... FIXME
             if maxGames is not None and gameCount >= maxGames:
                 break
@@ -691,11 +727,10 @@ class PGN:
             lineNumber += 1
             
         # Must be at least one game in the PGN file
-        if gameCount == 0:
+        if len(self.__games) == 0:
             raise Error('Empty PGN file')
 
         # Tidy up
-        gp.complete()
         p.endParse()
         f.close()
 
