@@ -2,19 +2,20 @@
 
 import os, os.path
 import errno
-import gettext
+import gettext 
 
-APP_DATA_DIR    = os.path.join('/usr/local', 'share') 
-ICON_DIR        = os.path.join(APP_DATA_DIR, 'pixmaps')
-TEXTURE_DIR     = os.path.join(ICON_DIR, 'glchess')
-GLADE_DIR       = os.path.join(APP_DATA_DIR, 'glchess')
-BASE_DIR        = os.path.join(APP_DATA_DIR, 'glchess')
-LOCALEDIR       = os.path.join(APP_DATA_DIR, 'locale')
-DATA_DIR        = os.path.expanduser('~/.gnome2/glchess/')
-LOG_DIR         = os.path.join(DATA_DIR, 'logs')
-CONFIG_FILE     = os.path.join(DATA_DIR, 'config.xml')
-AUTOSAVE_FILE   = os.path.join(DATA_DIR, 'autosave.pgn')
-LOCAL_AI_CONFIG = os.path.join(DATA_DIR, 'ai.xml')
+APP_DATA_DIR     = os.path.join('/usr/local', 'share') 
+ICON_DIR         = os.path.join(APP_DATA_DIR, 'pixmaps')
+TEXTURE_DIR      = os.path.join(ICON_DIR, 'glchess')
+GLADE_DIR        = os.path.join(APP_DATA_DIR, 'glchess')
+BASE_DIR         = os.path.join(APP_DATA_DIR, 'glchess')
+LOCALEDIR        = os.path.join(APP_DATA_DIR, 'locale')
+DATA_DIR         = os.path.expanduser('~/.gnome2/glchess/')
+LOG_DIR          = os.path.join(DATA_DIR, 'logs')
+CONFIG_FILE      = os.path.join(DATA_DIR, 'config.xml')
+HISTORY_DIR      = os.path.join(DATA_DIR, 'history')
+UNFINISHED_FILE  = os.path.join(HISTORY_DIR, 'unfinished')
+LOCAL_AI_CONFIG  = os.path.join(DATA_DIR, 'ai.xml')
 
 DOMAIN = 'gnome-games'
 gettext.bindtextdomain(DOMAIN, LOCALEDIR)
@@ -25,7 +26,7 @@ import gtk.glade
 gtk.glade.bindtextdomain(DOMAIN, LOCALEDIR)
 gtk.glade.textdomain(DOMAIN)
 
-VERSION   = "2.20.1"
+VERSION   = "2.21.1"
 APPNAME   = _("glChess")
 ICON_NAME = 'gnome-glchess'
 
