@@ -10,26 +10,13 @@ import glchess.chess
 import glchess.game
 
 # Optionally use OpenGL support
-haveGLSupport = True
 try:
+    import gtk.gtkgl
     import OpenGL.GL
 except:
     haveGLSupport = False
-try:
-    import gtk.gtkgl
-    import gtk.gdkgl
-except:
-    haveGLSupport = False
 else:
-    display_mode = (gtk.gdkgl.MODE_RGB | gtk.gdkgl.MODE_DEPTH | gtk.gdkgl.MODE_DOUBLE)
-    try:
-        glConfig = gtk.gdkgl.Config(mode = display_mode)
-    except gtk.gdkgl.NoMatches:
-        display_mode &= ~gtk.gdkgl.MODE_DOUBLE
-        try:
-            glConfig = gtk.gdkgl.Config(mode = display_mode)
-        except gtk.gdkgl.NoMatches:
-            haveGLSupport = False
+    haveGLSupport = True
 
 __all__ = ['GtkView']
 
@@ -67,7 +54,7 @@ class GtkViewArea(gtk.DrawingArea):
         
         # Make openGL drawable
         if hasattr(gtk, 'gtkgl'):
-            gtk.gtkgl.widget_set_gl_capability(self, glConfig)# FIXME:, share_list=glContext)
+            gtk.gtkgl.widget_set_gl_capability(self, self.view.ui.glConfig)# FIXME:, share_list=glContext)
 
         # Connect signals
         self.connect('realize', self.__init)
@@ -237,19 +224,18 @@ class GtkView(glchess.ui.ViewController):
     whiteTime        = None
     blackTime        = None
     
-    def __init__(self, ui, title, feedback, isActive = True, moveFormat = 'human', showComments = False):
+    def __init__(self, ui, title, feedback, moveFormat = 'human', showComments = False):
         """Constructor for a view.
         
         'feedback' is the feedback object for this view (extends ui.ViewFeedback).
-        'isActive' is a flag showing if this view can be controlled by the user (True) or not (False).
         'moveFormat' is the format name to display moves in (string).
         """
         self.ui = ui
         self.title = title
         self.feedback = feedback
-        self.isActive = isActive
         self.moveFormat = moveFormat
         self.showComments = showComments
+        self.hasFile = False
         
         # The GTK+ elements
         self.gui = gtkui.loadGladeFile('chess_view.glade', 'chess_view')
@@ -258,8 +244,7 @@ class GtkView(glchess.ui.ViewController):
         self.viewWidget = GtkViewArea(self)
         self.gui.get_widget('view_container').add(self.viewWidget)
 
-	if self.ui.tooltipStyle is not None:
-	    self.gui.get_widget('info_panel').set_style(self.ui.tooltipStyle)
+        self.ui.setTooltipStyle(self.gui.get_widget('info_panel'))
 
         # Make a model for navigation (move object, number, description) 
         model = gtk.ListStore(gobject.TYPE_PYOBJECT, int, str)
@@ -275,12 +260,6 @@ class GtkView(glchess.ui.ViewController):
         self.widget.show()
         self.viewWidget.show_all()
         
-    def _on_info_panel_expose_event(self, widget, event):
-        """Gtk+ callback"""
-        allocation = widget.allocation
-        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
-                                    allocation.x, allocation.y, allocation.width, allocation.height)
-
     def _on_comment_edit_button_toggled(self, widget):
         """Gtk+ callback"""
         label = self.gui.get_widget('panel_description_label')
@@ -389,15 +368,20 @@ class GtkView(glchess.ui.ViewController):
     def setWhiteTime(self, total, current):
         """Extends glchess.ui.ViewController"""
         self.whiteTime = (total, current)
-        if self.ui.notebook.getView() is self:
+        if self.ui.view is self:
             self.ui.setTimers(self.whiteTime, self.blackTime)
 
     def setBlackTime(self, total, current):
         """Extends glchess.ui.ViewController"""
         self.blackTime = (total, current)
-        if self.ui.notebook.getView() is self:
+        if self.ui.view is self:
             self.ui.setTimers(self.whiteTime, self.blackTime)
-
+            
+    def setHasFile(self, hasFile):
+        """Extends glchess.ui.ViewController"""
+        self.hasFile = hasFile
+        self.ui._updateViewButtons()
+            
     def setAttention(self, requiresAttention):
         """Extends glchess.ui.ViewController"""
         if self.requireAttention == requiresAttention:
@@ -411,21 +395,16 @@ class GtkView(glchess.ui.ViewController):
     def generateMoveString(self, move):
         """
         """
+        isWhite = move.number % 2 == 1
+        
         subs = {'movenum': (move.number - 1) / 2 + 1,
-                'move_can': move.canMove, 'move_san': move.sanMove,
+                'move_can': move.canMove,
+                'move_san': move.sanMove,
                 'piece': pieceNames[move.piece.getType()],
-                'start': move.start, 'end': move.end}
+                'start': move.start,
+                'end': move.end}
         if move.victim is not None:
             subs['victim_piece'] = pieceNames[move.victim.getType()]
-        subs['colour'] = _('White')
-        subs['victim_colour'] = _('Black')
-        if move.number % 2 == 1:
-            subs['short_colour'] = 'a'
-        else:
-            subs['short_colour'] = 'b'
-            t = subs['colour']
-            subs['colour'] = subs['victim_colour']
-            subs['victim_colour'] = t
         
         if self.moveFormat == 'san':
             if move.number % 2 == 0:
@@ -449,22 +428,36 @@ class GtkView(glchess.ui.ViewController):
         elif not move.opponentCanMove:
             status = _('Stalemate')
         if status is not None:
-            subs['suffix'] = _(' - %(check_status)s') % {'check_status': status}
-        else:
-            subs['suffix'] = ''
+            subs['result'] = status
+        haveResult = status is not None
+            
+        subs['move'] = {True:  _('%(movenum)2iw.') % subs,
+                        False: _('%(movenum)2ib.') % subs}[isWhite]
 
         if move.sanMove.startswith('O-O-O'):
-            string = _('%(movenum)2i%(short_colour)s. %(colour)s castles long%(suffix)s') % subs
+            string = {(True, True):   _('%(move)s White castles long (%(result)s)'),
+                      (True, False):  _('%(move)s White castles long'),
+                      (False, True):  _('%(move)s Black castles long (%(result)s)'),
+                      (False, False): _('%(move)s Black castles long')}[(isWhite, haveResult)]
         elif move.sanMove.startswith('O-O'):
-            string = _('%(movenum)2i%(short_colour)s. %(colour)s castles short%(suffix)s') % subs
+            string = {(True, True):   _('%(move)s White castles short (%(result)s)'),
+                      (True, False):  _('%(move)s White castles short'),
+                      (False, True):  _('%(move)s Black castles short (%(result)s)'),
+                      (False, False): _('%(move)s Black castles short')}[(isWhite, haveResult)]
         elif move.victim is not None:
-            string = _('%(movenum)2i%(short_colour)s. %(colour)s %(piece)s at %(start)s takes %(victim_colour)s %(victim_piece)s at %(end)s%(suffix)s') % subs
+            string = {(True, True):   _('%(move)s White %(piece)s at %(start)s takes the black %(victim_piece)s at %(end)s (%(result)s)'),
+                      (True, False):  _('%(move)s White %(piece)s at %(start)s takes the black %(victim_piece)s at %(end)s'),
+                      (False, True):  _('%(move)s Black %(piece)s at %(start)s takes the white %(victim_piece)s at %(end)s (%(result)s)'),
+                      (False, False): _('%(move)s Black %(piece)s at %(start)s takes the white %(victim_piece)s at %(end)s')}[(isWhite, haveResult)]
         else:
-            string = _('%(movenum)2i%(short_colour)s. %(colour)s %(piece)s moves from %(start)s to %(end)s%(suffix)s') % subs
+            string = {(True, True):   _('%(move)s White %(piece)s moves from %(start)s to %(end)s (%(result)s)'),
+                      (True, False):  _('%(move)s White %(piece)s moves from %(start)s to %(end)s'),
+                      (False, True):  _('%(move)s Black %(piece)s moves from %(start)s to %(end)s (%(result)s)'),
+                      (False, False): _('%(move)s Black %(piece)s moves from %(start)s to %(end)s')}[(isWhite, haveResult)]
 
         # FIXME: Promotion
 
-        return string
+        return string % subs
 
     def addMove(self, move):
         """Extends glchess.ui.ViewController"""        
@@ -505,15 +498,23 @@ class GtkView(glchess.ui.ViewController):
         elif game.rule is glchess.game.RULE_INSUFFICIENT_MATERIAL:
             if game.result is glchess.game.RESULT_DRAW:
                 description = _('Neither player can cause checkmate (insufficient material)')
-            else:
-                description = _('Opponent is unable to cause checkmate (insufficient material)')
+            elif game.result is glchess.game.RESULT_WHITE_WINS:
+                description = _('Black player is unable to cause checkmate (insufficient material)')
+            elif game.result is glchess.game.RESULT_BLACK_WINS:
+                description = _('White player is unable to cause checkmate (insufficient material)')
         elif game.rule is glchess.game.RULE_RESIGN:
-            description = _('One of the players has resigned')
+            if game.result is glchess.game.RESULT_WHITE_WINS:
+                description = _('The black player has resigned')
+            elif game.result is glchess.game.RESULT_BLACK_WINS:
+                description = _('The white player has resigned')
+            else:
+                assert(False)
         elif game.rule is glchess.game.RULE_DEATH:
             description = _('One of the players has died')
 
         self.gameResult = (title, description)
         self.updateInfoPanel()
+        self.ui._updateViewButtons()
     
     def close(self):
         """Extends glchess.ui.ViewController"""
