@@ -35,7 +35,9 @@ import glchess.config
 import glchess.ui
 import glchess.chess.board
 import dialogs
+import log
 import chessview
+import network
 
 # Mark all windows with our icon
 gtk.window_set_default_icon_name(ICON_NAME)
@@ -43,165 +45,6 @@ gtk.window_set_default_icon_name(ICON_NAME)
 def loadGladeFile(name, root = None):
     return gtk.glade.XML(os.path.join(GLADE_DIR, name), root, domain = DOMAIN)
 
-class GtkGameNotebook(gtk.Notebook):
-    """
-    """
-    
-    defaultView       = None
-    views             = None
-    viewsByWidget     = None
-    
-    def __init__(self, ui):
-        """
-        """
-        self.ui = ui
-        self.views = []
-        self.viewsByWidget = {}
-        
-        gtk.Notebook.__init__(self)
-        self.set_show_border(False)
-        
-        # Make the tabs scrollable so the area is not resized
-        self.set_scrollable(True)
-        
-        self.set_show_tabs(False)
-        
-    def setDefault(self, feedback):
-        """
-        """
-        assert(self.defaultView is None)
-        self.defaultView = chessview.GtkView(self.ui, '', feedback)
-        page = self.append_page(self.defaultView.widget)
-        self.set_current_page(page)
-        
-        self.__updateTabVisibleState()
-        
-        return self.defaultView
-
-    def addView(self, title, feedback):
-        """
-        """
-        moveFormat = glchess.config.get('move_format')
-        showComments = glchess.config.get('show_comments')
-        view = chessview.GtkView(self.ui, title, feedback, moveFormat = moveFormat, showComments = showComments)
-        self.views.append(view)
-        self.viewsByWidget[view.widget] = view
-        page = self.append_page(view.widget)
-        self.set_tab_label_text(view.widget, title)
-        self.set_current_page(page)
-        
-        self.__updateTabVisibleState()
-        
-        return view
-    
-    def getView(self):
-        """Get the currently selected view.
-        
-        Returns the view (GtkView) on this page or None if no view here.
-        """
-        # If splashscreen present then there is no view
-        if len(self.viewsByWidget) == 0:
-            return None
-        
-        num = self.get_current_page()
-        if num < 0:
-            return None
-        widget = self.get_nth_page(num)
-        return self.viewsByWidget[widget]
-    
-    def removeView(self, view):
-        """Remove a view from the notebook.
-        
-        'view' is the view to remove.
-        """
-        self.remove_page(self.page_num(view.widget))
-        self.views.remove(view)
-        self.viewsByWidget.pop(view.widget)
-        self.__updateTabVisibleState()
-
-    def __updateTabVisibleState(self):
-        """
-        """
-        # Only show tabs if there is more than one game
-        self.set_show_tabs(len(self.viewsByWidget) > 1)
-        
-        # Show/hide the default view
-        if len(self.viewsByWidget) == 0:
-            self.defaultView.widget.show()
-        else:
-            self.defaultView.widget.hide()
-
-class AIWindow:
-    """
-    """
-    
-    notebook    = None
-    defaultPage = None
-    
-    # We keep track of the number of pages as there is a bug 
-    # in GtkNotebook (Gnome bug #331785).
-    pageCount   = 0
-    
-    def __init__(self, notebook):
-        """
-        """
-        self.notebook = notebook
-        self.defaultPage = notebook.get_nth_page(0)
-    
-    def addView(self, title, executable, description):
-        """
-        """
-        # Hide the default page
-        self.defaultPage.hide()
-        self.notebook.set_show_tabs(True)
-        
-        self.pageCount += 1
-        return AIView(self, title, executable, description)
-
-class AIView:
-    """
-    """
-    
-    __gui = None
-    
-    def __init__(self, window, title, executable, description):
-        """
-        """
-        self.window = window
-        self.__gui = loadGladeFile('ai.glade', 'ai_table')
-        self.__gui.get_widget('executable_label').set_text(executable)
-        self.__gui.get_widget('game_label').set_text(description)
-
-        # Add into the notebook
-        self.root = self.__gui.get_widget('ai_table')
-        notebook = window.notebook
-        notebook.append_page(self.root, gtk.Label(title))
-                
-        # Create styles for the buffer
-        buffer = self.__gui.get_widget('comms_textview').get_buffer()
-        buffer.create_tag('input', family='Monospace')
-        buffer.create_tag('output', family='Monospace', weight = pango.WEIGHT_BOLD)
-        buffer.create_tag('move', family='Monospace', foreground = 'blue')
-        buffer.create_tag('info', family='Monospace', foreground = 'green')
-        buffer.create_tag('error', family='Monospace', foreground = 'red')
-
-    def addText(self, text, style):
-        """FIXME: Define style
-        """
-        buffer = self.__gui.get_widget('comms_textview').get_buffer()
-        buffer.insert_with_tags_by_name(buffer.get_end_iter(), text, style)
-        
-    def close(self):
-        """
-        """
-        self.window.pageCount -= 1
-        self.window.notebook.remove_page(self.window.notebook.page_num(self.root))
-        
-        # Show the default page
-        if self.window.pageCount == 0:
-            self.window.defaultPage.show()
-            self.window.notebook.set_show_tabs(False)
-            
 class GLibTimer(glchess.ui.Timer):
     """
     """
@@ -319,9 +162,6 @@ class GtkUI(glchess.ui.UI):
     __lastTime         = None
     __animationTimer   = None
     
-    # The notebook containing games
-    notebook           = None
-
     # The Gtk+ list model of the available player types
     __playerModel      = None
 
@@ -336,7 +176,8 @@ class GtkUI(glchess.ui.UI):
 
     # TODO
     __joinGameDialogs  = None
-
+    __networkGames     = None
+    
     __defaultWhiteAI   = None
     __defaultBlackAI   = None
 
@@ -350,6 +191,8 @@ class GtkUI(glchess.ui.UI):
     height             = None
     isFullscreen       = False
     isMaximised        = False
+    
+    view               = None
 
     def __init__(self, feedback):
         """Constructor for a GTK+ glChess GUI"""
@@ -358,8 +201,11 @@ class GtkUI(glchess.ui.UI):
                                properties={gnome.PARAM_APP_DATADIR: APP_DATA_DIR})
 
         self.feedback = feedback
+        self.__networkGames = {}
         self.__saveGameDialogs = {}
         self.__joinGameDialogs = []
+        
+        self._initOpenGL()
 
         # Set the message panel to the tooltip style
         # (copied from Gedit)
@@ -368,16 +214,14 @@ class GtkUI(glchess.ui.UI):
         tooltip.force_window()
         if hasattr(tooltip, 'tip_window') and tooltip.tip_window != None:
             tooltip.tip_window.ensure_style()
-            self.tooltipStyle = tooltip.tip_window.get_style()
+            self._tooltipStyle = tooltip.tip_window.get_style()
         else:
-            self.tooltipStyle = None
+            self._tooltipStyle = None
+        self._tooltipWidgetsDrawn = {}
         
         self._gui = loadGladeFile('glchess.glade')
         self._gui.signal_autoconnect(self)
         
-        # Workaround as Glade 2 always overrides the system style for toolbars
-        self.__getWidget('toolbar').unset_style()
-        
         # Create mappings between the promotion radio buttons and the promotion types
         self.__promotionTypeByRadio = {}
         self.__promotionRadioByType = {}
@@ -394,23 +238,13 @@ class GtkUI(glchess.ui.UI):
             self.__boardViewTypeByRadio[widget] = name
             self.__boardViewRadioByType[name] = widget
 
-        # Make a notebook for the games
-        self.notebook = GtkGameNotebook(self)
-        self.notebook.connect_after('switch-page', self._on_view_changed)
-        self.__getWidget('game_viewport').add(self.notebook)
-        self.notebook.show_all()
-        
         # Create the model for the player types
-        self.__playerModel = gtk.ListStore(str, gtk.gdk.Pixbuf, str)
+        self.__playerModel = gtk.ListStore(str, str, str)
         iconTheme = gtk.icon_theme_get_default()
-        try:
-            icon = iconTheme.load_icon('stock_people', 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-        except gobject.GError:
-            icon = None
         iter = self.__playerModel.append()
-        self.__playerModel.set(iter, 0, '', 1, icon, 2, _('Human'))
+        self.__playerModel.set(iter, 0, '', 1, 'stock_person', 2, _('Human'))
         
-        self.__aiWindow = AIWindow(self._gui.get_widget('ai_notebook'))
+        self.__logWindow = log.LogWindow(self._gui.get_widget('log_notebook'))
 
         # Balance space on each side of the history combo
         group = gtk.SizeGroup(gtk.SIZE_GROUP_BOTH)
@@ -423,26 +257,62 @@ class GtkUI(glchess.ui.UI):
         combo.pack_start(cell, False)
         combo.add_attribute(cell, 'text', 2)
 
-        self.defaultViewController = self.notebook.setDefault(None)
-
         # Disble help support
         if haveGnomeSupport:
             self._gui.get_widget('menu_help').show()
         
-        self.defaultViewController.viewWidget.setRenderGL(self.__renderGL)
-
         self.saveDialog = dialogs.SaveDialog(self)
         
+        self._updateViewButtons()
+        
         # Watch for config changes
         for key in ['show_toolbar', 'show_history', 'fullscreen',
                     'show_3d', 'show_comments', 'show_numbering',
                     'show_move_hints',
                     'width', 'height',
-                    'move_format', 'promotion_type', 'board_view']:
+                    'move_format', 'promotion_type', 'board_view',
+                    'enable_networking']:
             glchess.config.watch(key, self.__applyConfig)
+            
+    def _initOpenGL(self):
+        # Configure openGL
+        try:
+            gtk.gdkgl
+        except AttributeError:
+            self.glConfig = None
+        else:
+            display_mode = (gtk.gdkgl.MODE_RGB | gtk.gdkgl.MODE_DEPTH | gtk.gdkgl.MODE_DOUBLE)
+            try:
+                self.glConfig = gtk.gdkgl.Config(mode = display_mode)
+            except gtk.gdkgl.NoMatches:
+                display_mode &= ~gtk.gdkgl.MODE_DOUBLE
+                self.glConfig = gtk.gdkgl.Config(mode = display_mode)
 
     # Public methods
     
+    def setTooltipStyle(self, widget):
+        """Set a widget to be in the tooltip style.
+        
+        'widget' is the widget to modify.
+        """
+        if self._tooltipStyle is None:
+            return
+        widget.set_style(self._tooltipStyle)
+        widget.connect("expose_event", self._on_tooltip_expose_event)
+        widget.queue_draw()
+
+    def _on_tooltip_expose_event(self, widget, event):
+        """Gtk+ callback"""
+        allocation = widget.allocation
+        widget.style.paint_flat_box(widget.window, gtk.STATE_NORMAL, gtk.SHADOW_OUT, None, widget, "tooltip",
+                                    allocation.x, allocation.y, allocation.width, allocation.height)
+                                    
+        # The first draw is corrupt for me so draw it twice.
+        # Bonus points to anyone who tracks down the problem and fixes it
+        if not self._tooltipWidgetsDrawn.has_key(widget):
+            self._tooltipWidgetsDrawn[widget] = True
+            widget.queue_draw()
+    
     def watchFileDescriptor(self, fd):
         """Extends ui.UI"""
         gobject.io_add_watch(fd, gobject.IO_IN | gobject.IO_HUP, self.__readData)
@@ -474,32 +344,45 @@ class GtkUI(glchess.ui.UI):
         TODO: difficulty etc etc
         """
         iconTheme = gtk.icon_theme_get_default()
-        try:
-            icon = iconTheme.load_icon('stock_notebook', 24, gtk.ICON_LOOKUP_USE_BUILTIN)
-        except gobject.GError:
-            icon = None
         iter = self.__playerModel.append()
-        self.__playerModel.set(iter, 0, name, 1, icon, 2, name)
+        self.__playerModel.set(iter, 0, name, 1, 'stock_notebook', 2, name)
         
         # Get the human to play against this AI
         if self.__defaultBlackAI is None:
             self.__defaultBlackAI = name
-        
-    def setDefaultView(self, feedback):
-        """Extends ui.UI"""
-        self.defaultViewController.feedback = feedback
-        return self.defaultViewController
-        
-    def addView(self, title, feedback):
-        """Extends ui.UI"""
-        view = self.notebook.addView(title, feedback)
-        view.viewWidget.setRenderGL(self.__renderGL)
-        return view
 
-    def addAIWindow(self, title, executable, description):
+    def setView(self, title, feedback, isPlayable = True):
+        """Extends ui.UI"""
+        moveFormat = glchess.config.get('move_format')
+        showComments = glchess.config.get('show_comments')
+        self.view = chessview.GtkView(self, title, feedback, moveFormat = moveFormat, showComments = showComments)
+        self.view.isPlayable = isPlayable
+        self.view.viewWidget.setRenderGL(self.__renderGL)
+        viewport = self.__getWidget('game_viewport')
+        child = viewport.get_child()
+        if child is not None:
+            viewport.remove(child)
+        viewport.add(self.view.widget)
+        
+        # Set the window title to the name of the game
+        title = _('Chess')
+        if self.view is not None and len(self.view.title) > 0:
+            title += " - %s" % self.view.title
+        self._gui.get_widget('glchess_app').set_title(title)
+
+        # Set toolbar/menu buttons to state for this game
+        self._updateViewButtons()
+        
+        # Update timers
+        if self.view is not None:
+            self.setTimers(self.view.whiteTime, self.view.blackTime)
+
+        return self.view
+
+    def addLogWindow(self, title, executable, description):
         """
         """
-        return self.__aiWindow.addView(title, executable, description)
+        return self.__logWindow.addView(title, executable, description)
     
     def setTimers(self, whiteTime, blackTime):
         """
@@ -530,7 +413,8 @@ class GtkUI(glchess.ui.UI):
         # Load configuration
         for name in ['show_toolbar', 'show_history', 'show_3d',
                      'show_comments', 'show_numbering', 'show_move_hints',
-                     'move_format', 'promotion_type', 'board_view', 'maximised']:
+                     'move_format', 'promotion_type', 'board_view', 'maximised',
+                     'enable_networking']:
             try:
                 value = glchess.config.get(name)
             except glchess.config.Error:
@@ -555,6 +439,34 @@ class GtkUI(glchess.ui.UI):
         """Extends glchess.ui.UI"""
         dialogs.GtkNewGameDialog(self, self.__playerModel, game)
 
+    def addNetworkDialog(self, feedback):
+        """Extends glchess.ui.UI"""
+        # Create the dialog
+        dialog = network.GtkNetworkGameDialog(self, feedback, self.__playerModel)
+        self.__joinGameDialogs.append(dialog)
+        
+        # Add the detected games into the dialog
+        #for (game, name) in self.__networkGames.iteritems():
+        #    dialog.addNetworkGame(name, game)
+        
+        return dialog
+                                 
+    def addNetworkGame(self, name, game):
+        """Extends glchess.ui.UI"""
+        self.__networkGames[game] = name
+        
+        # Update the open dialogs
+        for dialog in self.__joinGameDialogs:
+            dialog.addNetworkGame(name, game)
+
+    def removeNetworkGame(self, game):
+        """Extends glchess.ui.UI"""
+        self.__networkGames.pop(game)
+
+        # Update the open dialogs
+        for dialog in self.__joinGameDialogs:
+            dialog.removeNetworkGame(game)
+        
     def close(self):
         """Extends glchess.ui.UI"""
         # Save the window size
@@ -593,14 +505,6 @@ class GtkUI(glchess.ui.UI):
             return error        
         self.__saveGameDialogs.pop(view)
 
-    def _removeView(self, view):
-        """Remove a view from the UI.
-        
-        'view' is the view to remove.
-        """
-        self.notebook.removeView(view)
-        self._updateViewButtons()
-
     # Private methods
 
     def __resize(self):
@@ -630,6 +534,16 @@ class GtkUI(glchess.ui.UI):
                 menu.set_active(False)
                 toolbar.hide()
                 
+        elif name == 'enable_networking':
+            menuItem = self.__getWidget('menu_play_online_item')
+            toolbarButton = self.__getWidget('play_online_button')
+            if value is True:
+                menuItem.show()
+                toolbarButton.show()
+            else:
+                menuItem.hide()
+                toolbarButton.hide()
+            
         # Show/hide the history
         elif name == 'show_history':
             box = self.__getWidget('navigation_box')
@@ -666,33 +580,26 @@ class GtkUI(glchess.ui.UI):
             self.__renderGL = value
             menuItem = self.__getWidget('menu_view_3d')
             menuItem.set_active(self.__renderGL)
-            self.notebook.defaultView.viewWidget.setRenderGL(self.__renderGL)
-            for view in self.notebook.views:
-                view.viewWidget.setRenderGL(self.__renderGL)
+            self.view.viewWidget.setRenderGL(self.__renderGL)
                 
         elif name == 'show_comments':
             menuItem = self.__getWidget('menu_view_comment')
             menuItem.set_active(value)
-            for view in self.notebook.views:
-                view.setShowComments(value)
+            self.view.setShowComments(value)
 
         elif name == 'show_move_hints':
             menuItem = self.__getWidget('menu_view_move_hints')
             menuItem.set_active(value)
-            for view in self.notebook.views:
-                view.feedback.showMoveHints(value)
+            self.view.feedback.showMoveHints(value)
 
         elif name == 'show_numbering':
             menuItem = self.__getWidget('menu_view_numbering')
             menuItem.set_active(value)
-            for view in self.notebook.views:
-                view.feedback.showBoardNumbering(value)
-            self.notebook.defaultView.feedback.showBoardNumbering(value)
+            self.view.feedback.showBoardNumbering(value)
 
         elif name == 'move_format':
             self._gui.get_widget('menu_movef_%s' % value).set_active(True)
-            for view in self.notebook.views:
-                view.setMoveFormat(value)
+            self.view.setMoveFormat(value)
                 
         elif name == 'promotion_type':
             try:
@@ -822,9 +729,9 @@ class GtkUI(glchess.ui.UI):
             value = False
         glchess.config.set('show_move_hints', value)       
 
-    def _on_show_ai_stats_clicked(self, widget):
+    def _on_show_logs_clicked(self, widget):
         """Gtk+ callback"""
-        window = self._gui.get_widget('ai_window')
+        window = self._gui.get_widget('log_window')
         if widget.get_active():
             window.show()
         else:
@@ -840,15 +747,21 @@ class GtkUI(glchess.ui.UI):
         # Get the move number
         moveNumber = model.get_value(iter, 1)
         
-        string = 'Show move number: ' + str(moveNumber)
+        string = 'Showing move number %d ' % moveNumber
         if moveNumber == len(model) - 1:
             string += ' (latest)'
             moveNumber = -1
-        
-        view = self.notebook.getView()
-        if view is not None:
-            view._setMoveNumber(moveNumber)
-            
+        print string
+
+        # Disable buttons when at the end
+        haveMoves = len(model) > 1
+        for widget in ('first_move_button', 'prev_move_button'):
+            self.__getWidget(widget).set_sensitive(haveMoves and moveNumber != 0)
+        for widget in ('last_move_button', 'next_move_button'):
+            self.__getWidget(widget).set_sensitive(haveMoves and moveNumber != -1)
+
+        self.view._setMoveNumber(moveNumber)
+
     def _on_menu_movef_human_activate(self, widget):
         """Gtk+ callback"""
         glchess.config.set('move_format', 'human')
@@ -918,74 +831,60 @@ class GtkUI(glchess.ui.UI):
         """Gtk+ callback"""
         self.__selectMoveNumber(-1)
 
-    def _on_view_changed(self, widget, page, pageNum):
-        """Gtk+ callback"""
-        # Set the window title to the name of the game
-        view = self.notebook.getView()
-        title = _('Chess')
-        if view is not None:
-            title += " - %s" % view.title
-        self._gui.get_widget('glchess_app').set_title(title)
-
-        # Set toolbar/menu buttons to state for this game
-        self._updateViewButtons()
-        
-        # Update timers
-        if view is not None:
-            self.setTimers(view.whiteTime, view.blackTime)
-    
     def _updateViewButtons(self):
         """
         """
-        view = self.notebook.getView()
-        enableWidgets = (view is not None) and view.isActive
-        self.__getWidget('end_game_button').set_sensitive(enableWidgets)
-        self.__getWidget('save_game_button').set_sensitive(enableWidgets)
-        self.__getWidget('menu_save_item').set_sensitive(enableWidgets)
-        self.__getWidget('menu_save_as_item').set_sensitive(enableWidgets)
-        self.__getWidget('menu_end_game_item').set_sensitive(enableWidgets)
+        enable = self.view is not None and self.view.isPlayable
+        for widget in ('save_game_button', 'menu_save_item', 'menu_save_as_item'):
+            self.__getWidget(widget).set_sensitive(enable)
 
         combo = self.__getWidget('history_combo')
-        if view is None:
+        if self.view is None:
             if combo.get_model() != None:
                 combo.set_model(None)
         else:
-            (model, selected) = view._getModel()
+            (model, selected) = self.view._getModel()
             combo.set_model(model)
             if selected < 0:
                 selected = len(model) + selected
             combo.set_active(selected)
-        self.__getWidget('navigation_box').set_sensitive(enableWidgets)
+        self.__getWidget('navigation_box').set_sensitive(enable)
+        
+        enable = enable and self.view.gameResult is None
+        for widget in ('menu_resign', 'resign_button', 'menu_claim_draw'):
+            self.__getWidget(widget).set_sensitive(enable)
 
     def _on_new_game_button_clicked(self, widget):
         """Gtk+ callback"""
         dialogs.GtkNewGameDialog(self, self.__playerModel)
 
+    def _on_join_game_button_clicked(self, widget):
+        """Gtk+ callback"""
+        self.feedback.onNewNetworkGame()
+
     def _on_open_game_button_clicked(self, widget):
         """Gtk+ callback"""
         dialogs.GtkLoadGameDialog(self)
         
     def _on_save_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        view = self.notebook.getView()
-        
-        if view.feedback.getFileName() is not None:
-            view.feedback.save()
-        elif not self.__saveGameDialogs.has_key(view):
-            self.__saveGameDialogs[view] = dialogs.GtkSaveGameDialog(self, view)
+        if self.view.feedback.getFileName() is not None:
+            self.view.feedback.save()
+        elif not self.__saveGameDialogs.has_key(self.view):
+            self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view)
             
     def _on_save_as_game_button_clicked(self, widget):
         """Gtk+ callback"""
-        view = self.notebook.getView()
-        if not self.__saveGameDialogs.has_key(view):
-            self.__saveGameDialogs[view] = dialogs.GtkSaveGameDialog(self, view, view.feedback.getFileName())
+        if not self.__saveGameDialogs.has_key(self.view):
+            self.__saveGameDialogs[self.view] = dialogs.GtkSaveGameDialog(self, self.view, self.view.feedback.getFileName())
 
-    def _on_end_game_button_clicked(self, widget):
+    def _on_resign_clicked(self, widget):
         """Gtk+ callback"""
-        view = self.notebook.getView()
-        assert(view is not None)
-        if view.feedback is not None:
-            view.feedback.close()
+        self.view.feedback.resign()
+
+    def _on_claim_draw_clicked(self, widget):
+        """Gtk+ callback"""
+        self.view.feedback.claimDraw()
 
     def _on_help_clicked(self, widget):
         """Gtk+ callback"""
@@ -1042,9 +941,9 @@ class GtkUI(glchess.ui.UI):
         self.__aboutDialog = None
         return False
         
-    def _on_ai_window_delete_event(self, widget, event):
+    def _on_log_window_delete_event(self, widget, event):
         """Gtk+ callback"""
-        self._gui.get_widget('menu_view_ai').set_active(False)
+        self._gui.get_widget('menu_view_logs').set_active(False)
         
         # Stop the event - the window will be closed by the menu event
         return True
@@ -1102,9 +1001,8 @@ class GtkUI(glchess.ui.UI):
     def _quit(self):
         # Check if any views need saving
         viewsToSave = []
-        for view in self.notebook.views:
-            if view.feedback.needsSaving():
-                viewsToSave.append(view)
+        if self.view.feedback.needsSaving():
+            viewsToSave.append(self.view)
 
         if len(viewsToSave) == 0:
             self.feedback.onQuit()
